#include <termio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <assert.h>

#include "modem.h"

tty::tty(const char *path)
{
  fd = open (path, O_RDWR | O_NOCTTY);
  assert (fd);
  reset ();
}

tty::~tty()
{
  close (fd);
}

void tty::mode_canon ()
{
  termios tio;
  assert (tcgetattr(fd, &tio) >= 0);
  cfsetospeed(&tio, B38400);
  cfsetispeed(&tio, B38400);
  tio.c_cflag |= (CRTSCTS);
  tio.c_iflag &= (IXON | IXOFF | IXANY);
  tio.c_oflag  = 0;
  tio.c_lflag  = ICANON;
//  tio.c_cc[VMIN]  = 1;
//  tio.c_cc[VTIME] = 100;
  assert (tcsetattr(fd, TCSANOW, &tio) >= 0);
}

void tty::flush ()
{
  tcflush (fd, TCIFLUSH);
}

void tty::reset ()
{
  flush ();
  mode_canon ();
}

void tty::write (const char *str)
{
  int len = strlen (str);
  
  assert (::write (fd, str, len) == len);
}

int tty::get_line (char *buff, int size)
{
  int len = read (fd, buff, size);
  buff[len] = 0;
  return len;
}

int tty::cmd (const char *cmd, char *buff, int size)
{
  int len;
  
  write (cmd);
  len = get_line (buff, size);
  if (buff[0] == '\r' || (cmd[0] == buff[0] && cmd[1] == buff[1])) // skip modem echo
    len = get_line (buff, size);
  
  return len;
}


