#ifndef UTIL_H
#define UTIL_H

#include <sys/time.h>
#include <sys/types.h>
#include <string>

#define L_JUNK	0x0001
#define L_ISDN	0x0002	// isdnctrl output
#define L_ITCP	0x0004	// itcp_connection
#define L_DISC	0x0008	// discovery protocol
#define L_PKT	0x0010	// RPKT/SPKT
#define L_LINE	0x0020	// transmitter/cuase codes

#define L_DEBUG	0x0040
#define L_INFO	0x0080
#define L_WARN	0x0100
#define L_ERROR	0x0200
#define L_FATAL	0x0400

extern int log_mask;

#define clog(l,c,f,x...) if ((l) & log_mask) printf("%ld, %s: " f, pthread_self(), c+strlen(c)-2 , ## x), fflush(stdout)
#define slog(l,f,x...) clog(l,src_msn,f , ## x)
#define log(l,f,x...) clog(l,"--",f , ## x)

typedef unsigned char u8;

#define abstract = 0

typedef long timeout;

// principal resolution: milliseconds
struct timestamp : private timeval {
  timestamp() { clear(); };
  void clear() { tv_sec = tv_usec = 0; };
  void stamp (timeout inc = 0) { gettimeofday (this, 0); tv_sec += inc / 1000; };
  bool past () { timeval tv; gettimeofday (&tv, 0);
                 return tv_sec < tv.tv_sec || (tv_sec == tv.tv_sec && tv_usec <= tv.tv_usec); };
  void age (timeout inc) { tv_sec -= inc / 1000; }; // verb: increase age
  timeout age () const; // substantive: how old
};

template<int start, int mul, int div, int max>
class exponential_timeout {
  timeout to;
public:  
  reset () { to = start; };
  exponential_timeout () { reset (); };
  next () { to = to * mul / div; if (to > max) to = start; };
  timeout operator ()() { return to; };
};

struct string_less_than
{
  bool operator()(const string &s1, const string &s2) const { return s1 < s2; }
};

void parse_logopt(const char *opts);

void init_debug (char **args);

#endif


